<project name="DeNovoPipeline" default="jarWithClasspathDebian" basedir=".">
	<description>
		DeNovoPipeline ant build file
	</description>

	<!-- set global properties for this build -->
	<!---Dusr=/usr/local-->

	<property name="usr.dir" location="@CMAKE_INSTALL_PREFIX@" />
	<property name="sys.dir" location="/usr/share/java" />
	<property name="share.dest" location="@CMAKE_BINARY_DIR@/share" />
	<property name="share" location="@CMAKE_SOURCE_DIR@/share" />
	<property name="lib" location="@CMAKE_SOURCE_DIR@/lib" />
	<property name="src" location="@CMAKE_SOURCE_DIR@/src" />
	<property name="build" location="@CMAKE_BINARY_DIR@/build" />
	<property name="dist" location="@CMAKE_BINARY_DIR@/dist" />
	<property name="jnlp" location="@CMAKE_BINARY_DIR@/jnlp" />
	<property name="confDir" location="@CMAKE_SOURCE_DIR@/src/resources/conf" />
	<property name="docDir" location="doc" />


	<!-- sign informations -->
	<property name="keystore" value="/home/langella/.keystore" />
	<property name="alias" value="your_alias_in_keystore" />
	<property name="storepass" value="your_password_for_storepass" />
	<property name="keypass" value="your_password_for_keypass" />


	<target name="reset">
		<!-- reset the build and dist directory -->
		<delete dir="${build}" />
		<delete dir="${dist}" />
		<mkdir dir="${build}" />
		<mkdir dir="${dist}" />

	</target>


	<target name="init" depends="reset">
		<!-- Create the time stamp -->
		<tstamp />
	</target>


	<path id="classpath">
		<fileset dir="${lib}" includes="**/*.jar" />
	</path>


	<target name="compile" depends="init" description="compile the source and put in build directory">
		<!-- Compile the java code from ${src} into ${build} -->
		<javac srcdir="${src}" destdir="${build}" target="1.6" source="1.6" classpathref="classpath" includeantruntime="false">
			<compilerarg line="-encoding utf-8" />
			<classpath>
				<pathelement location="${build}/fr/inra/pappso/DeNovo_pipeline.class" />
			</classpath>
		</javac>
	</target>


	<path id="classpathDebian">
		<!--xml-apis-ext.jar xml-apis.jar xercesImpl.jar xalan2.jar swt.jar batik.jar  liblog4j1.2-java -->
		<fileset dir="${sys.dir}" includes="xml-apis-ext.jar xml-apis.jar xercesImpl.jar xalan2.jar swt.jar batik.jar log4j-1.2.jar" />
		<fileset dir="${share.dest}/pepnovopipeline" includes="libproteomicsvg.jar" />
	</path>


	<target name="compileDebian" depends="init" description="compile the source and put in build directory">
		<!-- Compile the java code from ${src} into ${build} -->
		<copy file="${share}/pepnovopipeline/libproteomicsvg.jar" todir="${share.dest}/pepnovopipeline" overwrite="true" />
		<javac srcdir="${src}" destdir="${build}" target="1.6" source="1.6" classpathref="classpathDebian" includeantruntime="false">
			<compilerarg line="-encoding utf-8" />
			<classpath>
				<pathelement location="${build}/fr/inra/pappso/DeNovo_pipeline.class" />
			</classpath>
		</javac>
	</target>


	<target name="jarWithClasspathDebian" depends="compileDebian" description="generates the jar file for Debian">


		<copy todir="${build}/conf" overwrite="true">
			<fileset dir="${confDir}" includes="*" />
		</copy>
		<copy todir="${build}" overwrite="true">
			<fileset dir="${src}" includes="**" />
		</copy>

		<jar jarfile="${share.dest}/pepnovopipeline/DeNovo_pipeline.jar" basedir="${build}">
			<!--<fileset dir="${confDir}" />-->
			<!--<fileset file=".classpath" />-->
			<manifest>
				<attribute name="Main-Class" value="fr.inra.pappso.DeNovo_pipeline" />
				<attribute name="Class-Path" value="${usr.dir}/share/pepnovopipeline/libproteomicsvg.jar 
				${sys.dir}/xml-apis-ext.jar ${sys.dir}/xml-apis.jar ${sys.dir}/xercesImpl.jar ${sys.dir}/xalan2.jar
				 ${sys.dir}/batik.jar ${sys.dir}/log4j-1.2.jar" />
			</manifest>
		</jar>

		<!-- cd bin; jar -cf client.jar; -->
		<!-- <jar jarfile="client.jar" basedir="bin"/> -->
	</target>

	<target name="clean">
		<delete dir="${jnlp}" />
		<delete dir="${build}" />
		<delete dir="${dist}" />
	</target>

	<target name="dir.check">
		<condition property="dir.exists">
			<available file="@CMAKE_SOURCE_DIR@/lib" type="dir" />
		</condition>
	</target>

	<target name="copyInShare" description="copies jar needed by the debian package into the share directory" depends="dir.check" if="dir.exists">
		<copy file="@CMAKE_SOURCE_DIR@/lib/libproteomicsvg.jar" todir="${share}/pepnovopipeline" overwrite="true" />
	</target>


	<target name="jar" depends="compile" description="generates the jar file of the build directory">
		<mkdir dir="${build}/conf" />
		<copy todir="${build}/conf" overwrite="true">
			<fileset dir="${confDir}" includes="*" />
		</copy>

		<jar jarfile="${dist}/DeNovo_pipeline.jar" basedir="${build}">
			<!--<fileset file=".classpath" />-->
			<manifest>
				<attribute name="Main-Class" value="fr.inra.pappso.DeNovo_pipeline" />
			</manifest>
		</jar>


		<!-- cd bin; jar -cf client.jar; -->
		<!-- <jar jarfile="client.jar" basedir="bin"/> -->
	</target>


	<target name="generate_jnlp_dist" depends="jar" description="generate the MyXtandemParser application">

		<mkdir dir="${dist}/lib_signed" />

		<signjar jar="${dist}/DeNovo_pipeline.jar" keystore="${keystore}" alias="${alias}" storepass="${storepass}" keypass="${keypass}" signedjar="${dist}/sDeNovo_pipeline.jar" />
		<!-- <signjar jar="images.jar" keystore="/home/langella/.keystore" alias="langella" 
			storepass="toto" keypass="toto" signedjar="${dist}/simages.jar" /> <signjar 
			jar="user_guide.jar" keystore="/home/langella/.keystore" alias="langella" 
			storepass="toto" keypass="toto" signedjar="${dist}/suser_guide.jar" /> -->
		<signjar destDir="${dist}/lib_signed" keystore="${keystore}" alias="${alias}" storepass="${storepass}" keypass="${keypass}">
			<path>
				<fileset dir="${lib}" includes="*.jar" />
			</path>
		</signjar>


		<!-- construction du répertoire download pour JNLP-->


		<copy todir="${jnlp}/lib" overwrite="true">
			<fileset dir="${dist}/lib_signed" includes="*" />
		</copy>

		<copy file="${dist}/sDeNovo_pipeline.jar" tofile="${jnlp}/DeNovo_pipeline.jar" overwrite="true" />
		<copy file="./share/pepnovopipeline/pepnovopipeline.jnlp" tofile="${jnlp}/pepnovopipeline.jnlp" overwrite="true" />

	</target>

</project>
