package fr.inra.pappso.class_pep;

import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;

import fr.inra.moulon.svgutils.svgspectrum.*;
import fr.inra.pappso.pep_exception.PepException;
import fr.inra.pappso.pipeline.pepnovo_pip;

public class ions {
	private static final Logger logger = Logger.getLogger(ions.class);

	private Hashtable<String, Float> masses;

	private float proton = Float.valueOf("1.00794").floatValue();

	private float water = Float.valueOf("18.01099").floatValue();
	
	private float amine = Float.valueOf("17.02689").floatValue();
	
	private float co = Float.valueOf("27.99491").floatValue();
	
	
	private String sequence;

	private Vector<modifs> mods;
	
	private Vector<SvgIon> ionstable;

	public ions() {
		initialise();
		mods = new Vector<modifs> ();
		ionstable = new Vector<SvgIon> ();
	}

	public void setSequence(String sequence) {
		this.sequence = sequence;
	}

	public String getSequence() {
		return sequence;
	}

	public void setMods(Vector<modifs> mods) {
		this.mods = mods;
	}

	public Vector<modifs> getMods() {
		return mods;
	}

	private void initialise() {
		// TODO Auto-generated method stub
		masses = new Hashtable<String, Float>();
		masses.put("G", (float) 57.0214);
		masses.put("A", (float) 71.0371);
		masses.put("S", (float) 87.0320);
		masses.put("P", (float) 97.0527);
		masses.put("V", (float) 99.0684);
		masses.put("T", (float) 101.0476);
		masses.put("L", (float) 113.0840);
		masses.put("I", (float) 113.0840);
		masses.put("N", (float) 114.0429);
		masses.put("D", (float) 115.0269);
		masses.put("K", (float) 128.0949);
		masses.put("Q", (float) 128.0585);
		masses.put("E", (float) 129.0425);
		masses.put("M", (float) 131.0404);
		masses.put("m", (float) 147.0355); // METHIONINE OXIDEE (+16)
		//masses.put("m", (float) 131.0404);
		masses.put("H", (float) 137.0589);
		masses.put("F", (float) 147.0684);
		masses.put("R", (float) 156.1011);
		masses.put("C", (float) 161.0142); // CYSTEINE CARBAMIDOMETHYLE
		masses.put("Y", (float) 163.0633);
		masses.put("W", (float) 186.0793);
		masses.put("X", (float) 0);

	}

	public float get_MH() throws Exception {
		float mh = this.get_monoisotique_mass();
		mh += this.proton;
		return (mh);
	}

	private float get_monoisotique_mass() throws Exception {
		float mh = this.water;
		String aa = "";
		for (int i = 0; i < sequence.length(); i++) {
			aa = sequence.substring(i, i + 1);
			if (!masses.containsKey(aa))
				throw new PepException("Impossible de lire l'aa : " + aa);
			mh += masses.get(aa);
		}

		for (int j = 0; j < mods.size(); j++) {
			modifs m = mods.elementAt(j);
			mh += m.get_modvalue();
		}
		return (mh);
	}

	public Vector<SvgIon> get_ion_table() throws Exception{
		ionstable.clear();
		//ions de type y
		String aa = "";
		float somme = this.water+this.proton;
		logger.debug(sequence);
		//System.out.println(sequence);
		int position = 1;
		for (int i = sequence.length(); i > 0; i--) {
			aa = sequence.substring(i-1, i);
			if (!masses.containsKey(aa))
				throw new PepException("Impossible to read amino-acid : " + aa);
			somme += masses.get(aa);
			somme += this.get_mod_to_this_aa(i);
			//y+1
			SvgIon ion = new SvgIon();
			ion.setIonY1();
			ion.setMz(somme);
			ion.setNumber(position);
			ionstable.add(ion);
			//y+2
			ion = new SvgIon();
			ion.setIonY2();
			ion.setMz((somme+proton)/2);
			ion.setNumber(position);			
			ionstable.add(ion);
			//y+1*
			ion = new SvgIon();
			ion.setIonYstar1();
			ion.setMz(somme-amine);
			ion.setNumber(position);
			ionstable.add(ion);
			//y+2*
			ion = new SvgIon();
			ion.setIonYstar2();
			ion.setMz((somme-amine+proton)/2);
			ion.setNumber(position);			
			ionstable.add(ion);
			//y+1O
			ion = new SvgIon();
			ion.setIonYO1();
			ion.setMz(somme-water);
			ion.setNumber(position);
			ionstable.add(ion);
			//y+2O
			ion = new SvgIon();
			ion.setIonYO2();
			ion.setMz((somme-water+proton)/2);
			ion.setNumber(position);			
			ionstable.add(ion);
			
			logger.debug(somme);
			position++;
		}
		//ions de type b
		logger.debug("B");
		somme = this.proton;
		position = 1;
		for (int i = 0; i < sequence.length(); i++) {
			aa = sequence.substring(i, i + 1);
			if (!masses.containsKey(aa))
				throw new PepException("Impossible to read amino-acid : " + aa);
			somme += masses.get(aa);
			somme += this.get_mod_to_this_aa(i+1);
			//b+1
			SvgIon ion = new SvgIon();
			ion.setIonB1();
			ion.setMz(somme);
			ion.setNumber(position);
			ionstable.add(ion);
			//b+2
			ion = new SvgIon();
			ion.setIonB2();
			ion.setMz((somme+proton)/2);
			ion.setNumber(position);
			ionstable.add(ion);		
			//b+1*
			ion = new SvgIon();
			ion.setIonBstar1();
			ion.setMz(somme-amine);
			ion.setNumber(position);
			ionstable.add(ion);
			//b+2*
			ion = new SvgIon();
			ion.setIonBstar2();
			ion.setMz((somme-amine+proton)/2);
			ion.setNumber(position);			
			ionstable.add(ion);
			//b+1O
			ion = new SvgIon();
			ion.setIonBO1();
			ion.setMz(somme-water);
			ion.setNumber(position);
			ionstable.add(ion);
			//b+2O
			ion = new SvgIon();
			ion.setIonBO2();
			ion.setMz((somme-water+proton)/2);
			ion.setNumber(position);			
			ionstable.add(ion);
			//a
			ion = new SvgIon();
			ion.setIonA1();
			ion.setMz(somme-co);
			ion.setNumber(position);			
			ionstable.add(ion);
			
			position++;
		}
		return (ionstable);
	}
	
	private float get_mod_to_this_aa(int pos){
		for (int j = 0; j < mods.size(); j++) {
			modifs m = mods.elementAt(j);
			if (m.get_posi() == pos){
				logger.debug(m.get_formater_value());
				return m.get_modvalue();
			}
		}
		return(0);
	}
}
