package fr.inra.pappso.pipeline;

import java.io.File;
import java.io.Serializable;

import org.apache.log4j.Logger;

import fr.inra.pappso.pep_exception.PepException;

public class pepnovo_conf implements Serializable {
	private static final Logger logger = Logger.getLogger(pepnovo_conf.class);

	private static final long serialVersionUID = 121351L;

	float seuil;

	File xmlFile;

	File out_dir;

	File model_dir;

	File pepnovo_exec;

	String pepnovoModel = "CID_IT_TRYP";

	int thread;

	public pepnovo_conf() throws PepException {
		this.pepnovoModel = "CID_IT_TRYP";
		this.seuil = (float) 0.2;
		this.thread = 1;
		this.initialise();
	}

	private void initialise() throws PepException {
		// TODO Auto-generated method stub
		if (System.getProperty("os.name").contains("Windows")) {
			model_dir = new File("C:/Benperl/PepNovo_bin/Models");
			pepnovo_exec = new File("C:/Benperl/PepNovo_bin/PepNovo.exe");
		} else if (System.getProperty("os.name").contains("Linux")) {
			// model_dir = new File ("/opt/pepnovo/Models");
			// pepnovo_exec = new File("/opt/pepnovo/PepNovo_bin_ok");
			model_dir = new File("/usr/share/pepnovo/models");
			pepnovo_exec = new File("/usr/bin/pepnovo");
		} else
			throw new PepException(System.getProperty("os.name")
					+ " is not compatible with this java application");

		if (!pepnovo_exec.exists())
			throw new PepException("The PepNovo software : "
					+ pepnovo_exec.getAbsolutePath()
					+ " is not present on your system");

		if (!model_dir.exists())
			throw new PepException("The model directory : "
					+ model_dir.getAbsolutePath()
					+ " is not present on your system");

		logger.debug("New PepNovo config file");
	}

	public int getThread() {
		return thread;
	}

	public void setThread(int thread) {
		if (thread > 0)
			this.thread = thread;
	}

	public File getModel_dir() {
		return model_dir;
	}

	public void setModel_dir(File model_dir) throws PepException {
		this.model_dir = model_dir;
		if (!model_dir.exists())
			throw new PepException("The model directory : "
					+ model_dir.getAbsolutePath()
					+ " is not present on your system");

	}

	public File getPepnovo_exec() {
		return pepnovo_exec;
	}

	public void setPepnovo_exec(File pepnovo_exec) throws PepException {
		this.pepnovo_exec = pepnovo_exec;
		if (!pepnovo_exec.exists())
			throw new PepException("The PepNovo software : "
					+ pepnovo_exec.getAbsolutePath()
					+ " is not present on your system");
	}

	public float getSeuil() {
		return seuil;
	}

	public void setSeuil(float seuil) {
		this.seuil = seuil;
	}

	public File getXmlFile() {
		return xmlFile;
	}

	public void setXmlFile(File xtandemXmlFile) {
		this.xmlFile = xtandemXmlFile;
	}

	public File getOut_dir() {
		logger.debug("out_dir " + out_dir.getAbsolutePath());
		return out_dir;
	}

	public void setOut_dir(File out_dir) {
		logger.debug("set out_dir " + out_dir.getAbsolutePath());
		this.out_dir = out_dir;
	}

	public String getPepnovoModel() {
		if (pepnovoModel == null) {
			pepnovoModel = "CID_IT_TRYP";
		}
		if (pepnovoModel.isEmpty()) {
			pepnovoModel = "CID_IT_TRYP";
		}
		return this.pepnovoModel;
	}

	public void setPepnovoModel(String model) {
		this.pepnovoModel = model.trim();

	}

}
