package fr.inra.pappso.sax_parse;

import org.xml.sax.*;
import java.io.File;
import fr.inra.pappso.class_pep.Ms_run;
import fr.inra.pappso.class_pep.Spectre;
import fr.inra.pappso.pep_exception.PepException;

public class mzxml_parse extends org.xml.sax.helpers.DefaultHandler {
	private File sample_;

	private Spectre spectre_;

	private Spectrum_decode decoder_;

	private Ms_run msrun_;

	private String Last_element_;

	private String Start_element_;

	private String C_data;

	private float _RT;

	private int _scan;

	private float _mz;

	private int _charge;

	private int _peakscount;

	private int _mslevel;

	private String _byteOrder;

	private String _pairOrder;

	private int _precision;

	public mzxml_parse() {
		super();
	}

	public mzxml_parse(File S) {
		super();
		sample_ = S;
	}

	public void set_sample(File S) {
		sample_ = S;
	}

	// Les methodes qui suivent sont appelées
	// automatiquement par l'analyseur
	// lorsqu'un événement est détecté
	// dans le fichier XML.
	public void error(SAXParseException e) throws SAXParseException {
		throw (e);
	}

	public void startDocument() throws SAXException {
		msrun_ = new Ms_run(sample_);
		spectre_ = new Spectre();
		decoder_ = new Spectrum_decode();
		Last_element_ = "";
		Start_element_ = "";
		C_data = "";
		_scan = 0;
		_mz = 0;
		_charge = 0;
		_peakscount = 0;
		_RT = 0;
		_mslevel = 0;
		_precision = 0;
		_pairOrder = "";
		_byteOrder = "";
		// identification_.add_sample(sample_.getName());
	}

	public void endDocument() throws SAXException {
		// System.out.println("fin du document");
	}

	public void startElement(String namespaceURI, String simpleName,
			String qualifiedName, Attributes attrs) throws SAXException {
		Start_element_ = simpleName;
		if (Start_element_.equals(""))
			Start_element_ = qualifiedName;
		// System.out.println("Start= "+Start_element_);

		if (Start_element_.equals("scan")) {
			_scan = Integer.parseInt(attrs.getValue("num"));
			_peakscount = Integer.parseInt(attrs.getValue("peaksCount"));
			_mslevel = Integer.parseInt(attrs.getValue("msLevel"));

			String rt = attrs.getValue("retentionTime");
			rt = rt.replaceAll("PT", "");
			rt = rt.replaceAll("S", "");
			_RT = Float.valueOf(rt).floatValue();
		}

		if (Start_element_.equals("precursorMz")) {
			if (attrs.getValue("precursorCharge") != null)
				_charge = Integer.parseInt(attrs.getValue("precursorCharge"));
			else
				_charge = 0;
		}

		if (Start_element_.equals("peaks")) {
			_byteOrder = attrs.getValue("byteOrder");
			if (attrs.getValue("pairOrder") != null)
				_pairOrder = attrs.getValue("pairOrder");
			else
				_pairOrder = attrs.getValue("contentType");
			_precision = Integer.parseInt(attrs.getValue("precision"));
		}

		C_data = "";
	}

	public void endElement(String namespaceURI, String simpleName,
			String qualifiedName) throws SAXException {
		Last_element_ = simpleName;
		if (Last_element_.equals(""))
			Last_element_ = qualifiedName;

		if (Last_element_.equals("precursorMz")) {
			_mz = Float.valueOf(C_data).floatValue();
		}

		if (Last_element_.equals("peaks")) {
			// decodage spectre and ajout
			if ((_mslevel == 2) & (_peakscount != 0)) {
				spectre_.set_charge(_charge);
				spectre_.set_mz(_mz);
				spectre_.set_peak(_peakscount);
				spectre_.set_RT(_RT);
				spectre_.set_scan(_scan);
				spectre_.set_msrun(sample_);

				decoder_.set_byteorder(_byteOrder);
				decoder_.set_isPrecision64(_precision);
				try {
					decoder_.test_pair_order(_pairOrder);
					spectre_.set_spectra(decoder_.parseBinary(C_data));

					// ajout spectre au msrun
					msrun_.add_spectre(spectre_);

				} catch (PepException e) {
					throw new SAXException(e);
				} finally {
					spectre_ = new Spectre();
				}

			}

		}

		C_data = "";
	}

	public void characters(char buf[], int offset, int len) throws SAXException {
		C_data = C_data + new String(buf, offset, len);
		// System.out.println(C_data);
	}

	public Ms_run get_msruns() {
		return (msrun_);
	}

	public String get_sample() {
		return (sample_.getName());
	}
}
