package fr.inra.pappso.shell;

import java.util.Vector;
import java.util.regex.*;

import org.apache.log4j.Logger;

import fr.inra.pappso.class_pep.Fasts_result;
import fr.inra.pappso.class_pep.Ms_run;
import fr.inra.pappso.class_pep.Pep_result;

public class fasts_calculation extends base_shell {
	private static final Logger logger = Logger
			.getLogger(fasts_calculation.class);

	private Pattern result_bad;

	private Pattern result_ok;

	private Ms_run run;

	private Fasts_result result;

	private boolean valid_result;

	private boolean no_problem;

	private String save;

	private String sequences_matches;

	private String database;

	private String protein_seq;

	private boolean contaminant;

	public fasts_calculation() {
		super();
		result_bad = Pattern.compile("\\!\\! No sequences with E");
		result_ok = Pattern.compile("^\\>\\>*");

		result = new Fasts_result();
		valid_result = false;
		no_problem = false;
		contaminant = false;
		sequences_matches = "";
		protein_seq = "";
		save = "";
	}

	public void processing(String mess) {
		logger.debug(mess);
		Matcher resultat_ok = result_ok.matcher(mess);
		Matcher resultat_bad = result_bad.matcher(mess);
		if (resultat_ok.find()) {
			save += mess + "\n";

			mess = mess.replaceAll("^\\>\\>", "");
			mess = mess.replaceAll("\\(\\d+ aa\\)$", "");
			no_problem = true;
			valid_result = true;
			result.setDescription(mess);
			logger.debug(mess);

		} else if (resultat_bad.find()) {
			valid_result = false;
			no_problem = true;
			logger.debug(mess);
		}

		if ((valid_result)) {
			save += mess + "\n";
			String[] parse = mess.split("\\s+");
			if (parse.length > 1) {
				if (Pattern.matches("^Sample", parse[0]))
					sequences_matches += parse[1];
				if (result.getDescription().substring(0, 6).equals(parse[0]))
					protein_seq += parse[1];
			}
		}

	}

	public void set_Msrun(Ms_run run) {
		this.run = run;
	}

	public void setDatabase(String database) {
		this.database = database;
	}

	public String getDatabase() {
		return database;
	}

	public boolean isValid_result() {
		return valid_result;
	}

	public void test_result() {
		// System.out.println(this.get_Stdout());
		if (!no_problem)
			this.set_error("Error during fasts analysis\n" + this.get_Stdout());
		if (valid_result)
			save_fasts_result();
		// System.out.println(this.get_Stdout());
	}

	private void save_fasts_result() {
		// parsing pour ajouter le fasts_result
		// donnes sur la proteine

		Matcher match;
		int start = 0;
		logger.debug(save);
		match = Pattern
				.compile("E\\(\\d+\\)\\:\\s+(\\d+\\.?\\d?e?\\-?(\\d+)?)")
				.matcher(save);
		if (match.find())
			result.setEvalue(match.group(1));
		else
			this.set_error("Error getting Evalue");

		match = Pattern.compile("(\\d+\\.\\d+)\\% identity").matcher(save);
		if (match.find())
			result.setIdentity(Float.valueOf(match.group(1)).floatValue());
		else
			this.set_error("Error getting Identity");

		match = Pattern.compile("(\\d+\\.\\d+)\\% similar").matcher(save);
		if (match.find())
			result.setSimilarity(Float.valueOf(match.group(1)).floatValue());
		else
			this.set_error("Error getting similarity");

		match = Pattern.compile("in (\\d+) aa overlap").matcher(save);
		if (match.find())
			result.setOverlap(Integer.parseInt(match.group(1)));
		else
			this.set_error("Error geting overlap");

		match = Pattern
				.compile("Smith\\-Waterman score\\: (\\d+(\\.\\d+)?)\\;")
				.matcher(save);
		if (match.find())
			result.setScore(Float.valueOf(match.group(1)).floatValue());
		else
			this.set_error("Error getting Score");

		match = Pattern.compile("overlap \\(\\d+\\-\\d+\\:(\\d+)\\-\\d+\\)")
				.matcher(save);
		if (match.find())
			start = Integer.parseInt(match.group(1));
		else
			this.set_error("Error getting Start alignment");

		Vector<Pep_result> peps = run.get_Pepnovo_result();
		Pep_result pep;
		for (int i = 0; i < peps.size(); i++) {
			pep = peps.elementAt(i);
			match = Pattern.compile(pep.get_Sequence(),
					Pattern.CASE_INSENSITIVE).matcher(sequences_matches);

			if (match.find()) {
				result.add_Peptides(pep);
				result.add_Start("" + (match.start() + start));
				result.add_Stop(""
						+ (match.start() + match.group().length() + start - 1));

				logger.debug(match.group(0));
			}
		}
		result.setResults(save);
		result.setDatabase(database);
		result.setProtein_seq(protein_seq);
		result.setContaminant(contaminant);

		// verifier le nombre de peptides matchés, si aucun car seulement un
		// morceau, le résultat est non valide
		if (result.getscans().size() == 0)
			this.valid_result = false;
		else {
			run.add_Fasts_result(result);
		}
	}

	public void setContaminant(boolean contaminant) {
		this.contaminant = contaminant;
	}

	public boolean isContaminant() {
		return contaminant;
	}

}
